import edxml
from edxml.ontology import Brick, DataType


class EmailBrick(Brick):
    """
    Brick that defines some object types and concepts related to e-mail.
    """

    OBJECT_EMAIL_ADDRESS = 'computing.email.address'

    @classmethod
    def generate_object_types(cls, target_ontology):

        yield target_ontology.create_object_type(cls.OBJECT_EMAIL_ADDRESS)\
            .set_description('an RFC 5322 e-mail address') \
            .set_data_type(DataType.string(length=254))\
            .set_regex_soft(r'[\S]+@[\S]+\.[a-z]+')\
            .set_display_name('e-mail address', 'e-mail addresses')\
            .set_version(1)


edxml.ontology.Ontology.register_brick(EmailBrick)



import edxml
from edxml.ontology import Brick, DataType


class ComputingBrick(Brick):
    """
    Brick that defines some object types and concepts related to some generic
    concepts from computer science.
    """

    OBJECT_UUID = 'computing.identifier.uuid'
    OBJECT_OID = 'computing.identifier.oid'
    OBJECT_USER_NAME = 'computing.user.name'
    OBJECT_SOFTWARE_PRODUCT_NAME = 'computing.software.product.name'
    OBJECT_HARDWARE_PRODUCT_NAME = 'computing.hardware.product.name'
    OBJECT_SOFTWARE_VERSION = 'computing.software.product.version'
    OBJECT_OS_NAME = 'computing.software.os.name'
    OBJECT_CPE_URI = 'computing.cpe.uri'
    OBJECT_DATA_SIZE_BYTES = 'computing.data.size.bytes'

    CONCEPT_COMPUTER = 'entity.physical-entity.object.whole.artifact.instrumentality.device.machine.computer'
    CONCEPT_USER_ACCOUNT = 'entity.physical-entity.object.whole.living-thing.organism.person.user'

    @classmethod
    def generate_object_types(cls, target_ontology):

        yield target_ontology.create_object_type(cls.OBJECT_UUID) \
            .set_description('a universally unique identifier')\
            .set_data_type(DataType.uuid())\
            .set_display_name('UUID')\
            .set_xref('https://en.wikipedia.org/wiki/Universally_unique_identifier')\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_OID) \
            .set_description('an object identifier (OID) as standardized by '
                             'the International Telecommunications Union')\
            .set_data_type(DataType.string(255, upper_case=False, require_unicode=False))\
            .set_display_name('OID')\
            .set_regex_hard(r'[0-2]\.[\d]+(.\d+)*')\
            .set_xref('https://en.wikipedia.org/wiki/Object_identifier')\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_DATA_SIZE_BYTES) \
            .set_description('an amount of data measured in bytes')\
            .set_data_type(DataType.big_int(signed=False))\
            .set_unit('Byte', 'B')\
            .set_prefix_radix(2)\
            .set_display_name('size')\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_USER_NAME) \
            .set_description('a name of a computer user account')\
            .set_data_type(DataType.string(length=255)) \
            .set_display_name('user name')\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_SOFTWARE_PRODUCT_NAME) \
            .set_description('a name of a computer program') \
            .set_data_type(DataType.string(length=255, upper_case=False)) \
            .set_display_name('software product') \
            .fuzzy_match_phonetic()\
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_HARDWARE_PRODUCT_NAME) \
            .set_description('a name of a computer system or computer component') \
            .set_data_type(DataType.string(length=255)) \
            .set_display_name('hardware product') \
            .fuzzy_match_phonetic()\
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_SOFTWARE_VERSION) \
            .set_description('a version of a computer program') \
            .set_data_type(DataType.string(length=255, upper_case=False)) \
            .set_display_name('software version') \
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_OS_NAME) \
            .set_description('a name of a computer operating system') \
            .set_data_type(DataType.string(length=255, upper_case=False)) \
            .set_display_name('operating system')\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_CPE_URI)\
            .set_description('a Common Platform Enumeration (CPE) URI') \
            .set_data_type(DataType.uri(path_separator=':')) \
            .set_display_name('CPE URI')\
            .set_xref('https://en.wikipedia.org/wiki/Common_Platform_Enumeration')\
            .compress()\
            .set_version(1)

    @classmethod
    def generate_concepts(cls, target_ontology):

        yield target_ontology.create_concept(cls.CONCEPT_COMPUTER) \
            .set_description('a kind of a computing device') \
            .set_display_name('computer')\
            .set_version(1)

        yield target_ontology.create_concept(cls.CONCEPT_USER_ACCOUNT) \
            .set_description('a name of a user account on a computer system') \
            .set_display_name('user')\
            .set_version(1)


edxml.ontology.Ontology.register_brick(ComputingBrick)



import edxml
from edxml.ontology import Brick, DataType


class FilesBrick(Brick):
    """
    Brick that defines some object types and concepts related to computer files.
    """

    OBJECT_FILE_NAME = 'computing.file.name'
    OBJECT_FILE_NAME_CI = 'computing.file.name.ci'
    OBJECT_FILE_PATH = 'computing.file.path'
    OBJECT_FILE_PATH_CI = 'computing.file.path.ci'

    CONCEPT_FILE = 'entity.abstraction.communication.indication.evidence.record.file'

    @classmethod
    def generate_object_types(cls, target_ontology):

        yield target_ontology.create_object_type(cls.OBJECT_FILE_NAME) \
            .set_description('a name of a computer file')\
            .set_data_type(DataType.string(reverse_storage=True))\
            .set_display_name('file name')\
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_FILE_NAME_CI) \
            .set_description('a name of a computer file in a case insensitive file system') \
            .set_data_type(DataType.string(reverse_storage=True, lower_case=False))\
            .set_display_name('file name')\
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_FILE_PATH) \
            .set_description('a location on a computer filesystem') \
            .set_data_type(DataType.string(reverse_storage=True))\
            .set_display_name('path')\
            .compress()\
            .set_version(1)

        yield target_ontology.create_object_type(cls.OBJECT_FILE_PATH_CI) \
            .set_description('a location on a case insensitive computer filesystem') \
            .set_data_type(DataType.string(reverse_storage=True, lower_case=False))\
            .set_display_name('path')\
            .compress()\
            .set_version(1)

    @classmethod
    def generate_concepts(cls, target_ontology):
        yield target_ontology.create_concept(cls.CONCEPT_FILE) \
            .set_description('a computer resource for recording data') \
            .set_display_name('file')


edxml.ontology.Ontology.register_brick(FilesBrick)



from setuptools import setup, find_namespace_packages

# Explicitly state a version to please flake8
__version__ = 1.0
# This will read __version__ from edxml/version.py
exec(open('edxml_bricks/version.py').read())

setup(
    name='edxml-bricks-computing',
    version=__version__,

    # A description of your project
    description='EDXML ontology brick (computing)',
    long_description='EDXML ontology brick defining ontology elements related to computer science',

    # The project's main homepage
    # url='https://github.com/user/project',

    # Author details
    author='Dik Takken',
    author_email='dik.takken@edxml.org',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3'
    ],

    # What does your project relate to?
    keywords='edxml',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_namespace_packages(include=['edxml_bricks.*']),

    # List run-time dependencies here. These will be installed by pip when your
    # project is installed.
    # See https://pip.pypa.io/en/latest/reference/pip_install.html#requirements-file-format
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/technical.html#install-requires-vs-requirements-files
    install_requires=['edxml>=3.0.0.dev0'],
    extras_require={'dev': ['flake8', 'pytest', 'coverage']},
)
